# ICFP 2023 Artifact

Name: **Generic Programming with Extensible Data Types; Or, Making Ad Hoc Extensible Data Types Less Ad Hoc**


# Artifact Instructions

We structure this artifact and README.md according to the [instructions provided
by the ICFP'23
AEC](https://icfp23.sigplan.org/track/icfp-2023-artifact-evaluation#Submission-Guidelines). These
instructions are fairly strict on how to specify installation steps, but less so
for the provision of evaluation instructions. We found it suitable to adopt
the [evaluation instructions provided by the POPL'22
AEC](https://popl22.sigplan.org/track/POPL-2022-artifact-evaluation#documentation).

## Brief Artifact Description

Our artifact is an Agda library rooted under [./ROmega](./ROmega). We also
provide HTML documentation under [./html/](./html/). The development consists
principally of the following components:

- **An intrinsic mechanization of Rω**, the calculus introduced by §4. By
  "intrinsic", we mean that untyped terms and unkinded types have no meaning. So
  our ASTs are of _kinding and typing derivations_, not of _types_ and _terms_,
  respectively. The syntax and semantics of each judgment introduced in §4 are
  modularly housed in their own folders—e.g.,
  [ROmega.Types.Syntax](ROmega/Types/Syntax.agda) gives the AST for kinding
  derivations. For a full description of each module, see the [Library
  Structure](./README.md#Library-Structure) below.
- **A formalization of the Index Calculus**, which we give
  as semantics for Rω (§5). This is housed in
  [ROmega.IndexCalculus](./ROmega/IndexCalculus).
- **A Denotation of Rω into the Index Calculus**. This is housed in each Rω
  component folder's `Semantics.hs`, e.g. the denotation of types is given in
  [ROmega.Types.Semantics](./ROmega/Types/Semantics.agda).


A full description of each module is given
[below](./README.md#Library-Structure).

# Download, Installation, and Sanity-Testing

As per [ICFP'23 AEC
instructions](https://icfp23.sigplan.org/track/icfp-2023-artifact-evaluation#Submission-Guidelines),
we offer our artifact as both source and virtual image. Evaluators who have
working versions of Agda already installed on their machines are encouraged to
build from source, as the development has only the Agda standard library as
dependency.


## Building From Source

- If not done so already, install Agda according to the instructions relevant
  for your operating system. [Instructions
  here](https://agda.readthedocs.io/en/latest/getting-started/installation.html).
- As per Step 4 of the Agda installation instructions, you must **install the
  Agda Standard Library**. Instructions
  [here](https://github.com/agda/agda-stdlib/blob/master/notes/installation-guide.md).
- The development is housed under
[./ROmega/](./ROmega/). [ROmega.All](./ROmega/All.agda) imports **all**
modules; you may thus verify that all files compile by type checking `All.agda`.

The artifact was built & written using **Agda version 2.6.2.2** and the **Agda
standard library version 1.7.1**. As of this writing, the latest versions are
2.6.2.4 and 1.7.2, respectively. Our development will work (and has been tested)
with these latest versions.


## Building the Virtual Image
The image provided (`disk.qcow`) has Agda, the Agda standard library, and the
ROmega package installed. Please refer to the QEMU instructions at the bottom of
this README.

If you use the VM as a CLI, please run `agda` commands with
`/home/artifact/ROmega-ICFP23` as root directory. For example, to compile all
files, ssh into the VM and run


```sh
$ cd /home/artifact/ROmega-ICFP23
$ agda ROmega/All.agda 
```

This helps Agda resolve relative module locations. Compilation of the above
confirms that the VM has successfully built the ROmega library (but may take a
bit of time).


# List of Claims

We claim the following list of contributions in our introduction (§1). By
request from reviewers, our revised submission differs from our original
submission on Claim (3); we give below the revised version, which may differ
from your attached pdf. The revision is much weaker than the claim originally
made, and should be easier to verify.

## Claim (1)
> The extension of Rose to encompass generic programming over rows (§3),
> particularly the design of combinators that express generic transformations of
> row-typed products and sums;

## Claim (2)
> A formalization of our approach in the Rω calculus (§4), which extends System
> Fω with Rose-style row typing, first-class labels, and generic programming
> over rows; and,

## Claim (3)
> The denotation of Rω derivations in Agda (§5), showing Rω sound and
> demonstrating that Rω need not introduce additional runtime manipulation or
> comparison of labels and label sets.


# Evaluation Instructions

## Evaluating Claim (1)
This claim is bolstered by the paper, not the artifact.

## Evaluating Claim (2)

> A formalization of our approach in the Rω calculus (§4), 

See [Library Structure](./README.md#Library-Structure) below for a full
description of each module. In brief: The Rω calculus has

- **Kinds**, declared in [ROmega.Kinds.Syntax](./ROmega/Kinds/Syntax.agda)
  (§4, Figure 8).
- **Types**, declared (as kinding judgments) in
  [ROmega.Types.Syntax](./ROmega/Types/Syntax.agda) (§4, Figure 9).
- **Terms**, declared (as typing judgments) in
  [ROmega.Terms.Syntax](./ROmega/Terms/Syntax.agda) (§4, Figure 12).
- An **equivalence relation**, declared in
  [ROmega.Equivalence.Syntax](./ROmega/Equivalence/Syntax.agda) (§4, Figure 10).
- A **predicate entailment relation**, declared in
  [ROmega.Entailment.Syntax](./ROmega/Entailment/Syntax.agda) (§4, Figure 11).

We argue that each of the above ASTs constitute a "formalization of our approach
in the Rω Calculus."  The evaluator may skim each of the above modules,
particularly the declaration of judgment ASTs (e.g. `Type` in
`Types.Syntax.Agda`), to verify parity with the judgments of the paper.

> which extends System Fω with Rose-style row typing, 

This follows from our addition of rows to the [kind
syntax](./ROmega/Kinds/Syntax.agda). 

```agda
data Kind where
  ...
  R[_] : ∀ {ℓ} → Kind ℓ → Kind ℓ
```

The kind `R[ κ ]` denotes a row mapping labels to types of kind κ.  "Rose-Style"
is in reference to [Morris & McKinna
2019](https://homepage.cs.uiowa.edu/~jgmorrs/pubs/morris-popl2019-rows.pdf),
which describes the Rose type system. By "Rose-style" we refer specifically to
the universal quantification of row type variables and their qualification in
containment & combinement predicates. This follows from the addition of
universal quantification in typing judgments (see the [type
syntax](./ROmega/Types/Syntax.agda)):

```agda
data Type where
  ...	
  `∀ :  ∀ {ℓ₁ ℓ₂ ℓ₃ : Level} {Δ : KEnv ℓ₁} →
          (κ : Kind ℓ₃) → Type (Δ , κ) (★ ℓ₂) →
          -------------------------------------
          Type Δ (★ (ℓ₂ ⊔ (lsuc ℓ₃)))
```

and the qualification of types by predicates:
```agda
data Type where
  ...
  _⇒_ : ∀ {ℓ ℓκ ℓτ} {κ : Kind ℓκ} {Δ : KEnv ℓ}
          → (π : Pred Δ κ) → Type Δ (★ ℓτ) →
         --------------------------------
         Type Δ (★ (lsuc ℓκ ⊔ ℓτ))
```


> first-class labels, 

This follows from our addition of labels to the [kind
syntax](./ROmega/Kinds/Syntax.agda). 

```agda
data Kind where
  ...
  L : ∀ {ℓ} → Kind ℓ
```

As shown above, types may quantify over type variables of arbitrary kind,
including label kind. To make labels "first class", they must have a term-level
representation. This may be done by casting labels from kind `L` to kind ⋆ (in
the [type syntax](./ROmega/Types/Syntax.agda)) using the singleton type
constructor ⌊_⌋.

```agda
data Type where
  ...
  ⌊_⌋ : ∀ {ℓ ι : Level} {Δ : KEnv ℓ} →
         Type Δ L →
         ----------
         Type Δ (★ ι)
```

See §3.1 for example usage and further description.

> and generic programming over rows

Interpreting Rω under different row theories amounts to varying the syntax and
semantics of entailment (see [Morris & McKinna
2019](https://homepage.cs.uiowa.edu/~jgmorrs/pubs/morris-popl2019-rows.pdf)).
Thus, our development is generically parameterized by the syntax and semantics
of entailment. Observe that the syntax and semantics of simple rows are given
(resp.) as `SimpleRowSyntax` in
[ROmega.Entailment.Syntax](./ROmega/Entailment/Syntax.agda) and as
`SimpleRowSemantics` in
[ROmega.Entailment.Semantics](./ROmega/Entailment/Semantics.agda). The syntax
and semantics modules of terms are parameterized by the shape of entailment,
i.e., in [ROmega.Terms.Syntax](./ROmega/Terms/Syntax.agda), the `TermSyntax` is
parameterized by the syntax of entailment.

```agda
module TermSyntax 
  (Ent : 
    ∀ {ℓΔ ℓΦ ℓκ}
      {κ : Kind ℓκ}
      (Δ : KEnv ℓΔ) → PEnv Δ ℓΦ → Pred Δ κ → Set) where
```

Adoption of different row theories can be realized by instantiating entailment
with a different row theory AST and interpretation function. Hence we assert the
development is generic over different "row theories" (as defined in [Morris & McKinna
2019](https://homepage.cs.uiowa.edu/~jgmorrs/pubs/morris-popl2019-rows.pdf)).

## Evaluating Claim (3)


> The denotation of Rω derivations in Agda (§5)

Each AST is denoted into Agda in the corresponding `Semantics.agda` file. We ask
the evaluator to verify that the definitions of each interpretation function
(written as 〚_〛 followed by a suffix) are unsuprising.

- **Kinds** are denoted in [ROmega.Kinds.Semantics](./ROmega/Kinds/Semantics.agda): Star
  kinds are denoted into Agda `Set`s; row kinds are denoted into Index Calculus
  rows; label kinds are denoted into the unit type; and arrow kinds are denoted
  as Agda functions.
- **Kinding Judgments** at kind κ are denoted into the meaning of κ.
  
  ```agda 
  〚_〛 : Type Δ κ → ⟦ Δ ⟧ke → ⟦ κ ⟧k 
  ```
  ([.ROmega.Types.Semantics](./ROmega/Types/Semantics.agda)).
- **Typing judgments** at type τ are denoted into the meaning of τ
  ([./ROmega.Terms.Semantics](./ROmega/Terms/Syntax.agda)).
- Inhabitants of the **equivalence relation** denote into propositional equality.

  ```agda
  ⟦_⟧eq : ∀ {τ υ : Type Δ κ} →
       τ ≡t υ → (H : ⟦ Δ ⟧ke) → ⟦ τ ⟧t H ≡ ⟦ υ ⟧t H
  ```
  ([ROmega.Equivalence.Semantics](./ROmega/Equivalence/Semantics.agda)).
- Inhabitants of the **predicate entailment relation** denote into evidence in the index calculus.
  This is given greater explanation in §5.3.
  [ROmega.Entailment.Semantics](./ROmega/Entailment/Semantics.agda).

> showing Rω sound 

We claim soundness as consequence of the denotations (above) and the totality of
Agda, i.e, the denotations of well-kinded types are in the denotations of
their kinds,

```agda
⟦_⟧t : ∀ {ℓ ι : Level} {Δ : KEnv ℓ} {κ : Kind ι} →
      Type Δ κ → ⟦ Δ ⟧ke → ⟦ κ ⟧k
```

the denotations of well-typed terms are in the denotations of their types,


```agda
⟦_⟧ : ∀ {ℓΔ} {Δ : KEnv ℓΔ} {ℓΦ ℓΓ ℓτ} {Φ : PEnv Δ ℓΦ} {Γ : Env Δ ℓΓ}
        {τ : Type Δ (★ ℓτ)} →
        Term Δ Φ Γ τ →
        (H : ⟦ Δ ⟧ke) → ⟦ Φ ⟧pe H → ⟦ Γ ⟧e H → ⟦ τ ⟧t H
```

and so forth.  Because our denotations are in a typed theory, we do not have a
"wrong" value at all; instead, we extend the guarantees provided by Agda's type
system to Rω. Notably, as Agda is total, so are our interpretations, and thus all
derivations indeed have well-typed denotations.


> and demonstrating that Rω need not introduce additional runtime manipulation
> or comparison of labels and label sets.

This follows directly from the erasure of labels when interpreting Rω types and
kinds into the Index Calculus. Observe that the label kind is interpreted as the
level-polymorphic unit type ⊤ (in [ROmega.Kinds.Semantics](./ROmega/Kinds/Semantics.agda)):

```agda
⟦_⟧k : ∀ {ℓ} → Kind ℓ → Set (lsuc ℓ)
...
⟦ L ⟧k = ⊤

```

the label type is interpreted as the unit term `tt` (in [ROmega.Type.Semantics](./ROmega/Type/Semantics.agda)):

```agda
⟦_⟧t : ∀ {ℓ ι : Level} {Δ : KEnv ℓ} {κ : Kind ι} →
      Type Δ κ → ⟦ Δ ⟧ke → ⟦ κ ⟧k
...
⟦ lab l ⟧t       H = tt
```

and terms with label type are interpreted as the unit term `tt` (in [ROmega.Terms.Semantics](./ROmega/Terms/Semantics.agda)):

```agda
⟦_⟧ : ∀ {ℓΔ} {Δ : KEnv ℓΔ} {ℓΦ ℓΓ ℓτ} {Φ : PEnv Δ ℓΦ} {Γ : Env Δ ℓΓ}
        {τ : Type Δ (★ ℓτ)} →
        Term Δ Φ Γ τ →
        (H : ⟦ Δ ⟧ke) → ⟦ Φ ⟧pe H → ⟦ Γ ⟧e H → ⟦ τ ⟧t H
...		  
⟦ lab s ⟧ H φ η  = tt
```

if labels are not present at runtime, they cannot be compared or manipulated.

# Additional Artifact Description

## Example Derivations

The following examples of the paper were mechanized in
[ROmega/Examples/Section-3](./ROmega/Examples/Section-3.agda). The
evaluator is encouraged to use this file as a sandbox.

| Section | Paper Location | Line Location |
| :-:     |    :-:                                  | :--: |
| \S3.1   | `sel`, in ¶.                            | 58 |
| \S3.1   | `con`, in ̆¶.                            | 80 |
| \S3.1   | `case`, in ¶.                           | 114 |
| \S3.1   | `ifte`, in ¶.                           | 147 |
| \S3.2   | `reify` and `reflect`, in Figure 3.       | 175, 229 |
| \S3.2   | `map-π` and `map-Σ`, in figure 4.       | 331, 396 |


## Library Structure

We offer a full description of each folder and file, for reference.

[ROmega.All](./ROmega/All.agda) imports **all** modules. Files and folders
are named as the Agda std-lib does, e.g., `ROMega.Foobar` simply
exports the contents of the (base) files within `ROmega.Foobar.*`. Theorems
and lemmas of module `Foobar` are named `Foobar.Properties.agda`.

### ./IndexCalculus/

System Rω _denotes into_ a subset of Agda which we term the Index Calculus (see
§5). So, this directory houses the semantic image of Rω.

- [ROmega.IndexCalculus.Rows](./ROmega/IndexCalculus/Rows.agda). Definition of
  rows and row evidence.
- [ROmega.IndexCalculus.Records](./ROmega/IndexCalculus/Records.agda). Definition
  of records.
- [ROmega.IndexCalculus.Variants](./ROmega/IndexCalculus/Variants.agda). Definition
  of variants.
- [ROmega.IndexCalculus.Properties](./ROmega/IndexCalculus/Properties.agda). Properties
  of rows -- in particular, that `ρ pick i` and `ρ delete i` can be recombined
  into `ρ`.
  
### ./Postulates/

We postulate functional extensionality in
[ROmega.Postulates.FunExt](./ROmega/Postulates/FunExt.agda).

### ./Lib/

Common utility is placed in [ROmega.Lib](./ROmega/Lib). We define `≡-elim` in
[.ROmega.Lib.Equality](./ROmega/Lib/Equality.agda), which we use when rewriting
is inconvenient or not possible (e.g., in a let statement).


### ./Kinds/
- [ROmega.Kinds.Syntax](./ROmega/Kinds/Syntax.agda). The syntax of level
  stratified kinds.
- [ROmega.Kinds.Semantics](./ROmega/Kinds/Semantics.agda). The denotation
  of Rω kinds into Agda types.

### ./Types/

- [ROmega.Types.Syntax](./ROmega/Types/Syntax.agda). The
  intrinsically-kinded AST representation of (stratified) kinding derivations.
- [ROmega.Types.Semantics](./ROmega/Types/Semantics.agda). Denotation of
  kinding derivations into Agda.
- [ROmega.Types.Substitution](./ROmega/Types/Substitution.agda). Definition
  of renaming & substitution. This is necessary because, although we denote Rω
  _terms_ into Agda, we still have (operational) β-reduction at the type level.
- [ROmega.Types.Substitution.Properties](./ROmega/Types/Substitution/Properties.agda). Weakening
  & substitution lemmas for type-level β-reduction.

### ./Terms/

- [ROmega.Terms.Syntax](./ROmega/Terms/Syntax.agda). The
  intrinsically-typed AST representation of typing derivations.
- [ROmega.Terms.Semantics](./ROmega/Terms/Semantics.agda). Denotation of
  typing derivations into Agda.

### ./Equivalence/

- [ROmega.Equivalence.Syntax](./ROmega/Equivalence/Syntax.agda). The AST of
  type (and predicate) equivalence. 
- [ROmega.Equivalence.Semantics](./ROmega/Equivalence/Semantics.agda). Denotation
  of type equivalence into propositional equality in Agda.

### ./Entailment/

- [ROmega.Entailment.Syntax](./ROmega/Entailment/Syntax.agda). The AST
  of predicate entailment.
- [ROmega.Entailment.Semantics](./ROmega/Entailment/Semantics.agda). Denotation
  of predicate entailment into evidence in the Index Calculus.
- [ROmega.Entailment.Reasoning](./ROmega/Entailment/Reasoning.agda). Helper library, á la the
  [std-lib](https://github.com/agda/agda-stdlib/blob/master/src/Relation/Binary/PropositionalEquality/Core.agda#L103),
  for constructing inhabitants of the entailment relation.

### ./Examples/

- [ROmega.Examples.Section-3](./ROmega/Examples/Section-3.agda). Example
  typing derivations indexed with respect to their placement in section 3 of the
  text

# QEMU Instructions

**The following instructions are provided by the ICFP 2023 AEC.**

The ICFP 2023 Artifact Evaluation Process is using a Debian QEMU image as a
base for artifacts. The Artifact Evaluation Committee (AEC) will verify that
this image works on their own machines before distributing it to authors.
Authors are encouraged to extend the provided image instead of creating their
own. If it is not practical for authors to use the provided image then please
contact the AEC co-chairs before submission.

QEMU is a hosted virtual machine monitor that can emulate a host processor
via dynamic binary translation. On common host platforms QEMU can also use
a host provided virtualization layer, which is faster than dynamic binary
translation.

QEMU homepage: https://www.qemu.org/

### Installation

#### OSX
``brew install qemu``

#### Debian and Ubuntu Linux
``apt-get install qemu-kvm``

On x86 laptops and server machines you may need to enable the
"Intel Virtualization Technology" setting in your BIOS, as some manufacturers
leave this disabled by default. See Debugging.md for details.


#### Arch Linux

``pacman -Sy qemu``

See the [Arch wiki](https://wiki.archlinux.org/title/QEMU) for more info.

See Debugging.md if you have problems logging into the artifact via SSH.


#### Windows 10

Download and install QEMU via the links at

https://www.qemu.org/download/#windows.

Ensure that `qemu-system-x86_64.exe` is in your path.

Start Bar -> Search -> "Windows Features"
          -> enable "Hyper-V" and "Windows Hypervisor Platform".

Restart your computer.

#### Windows 8

See Debugging.md for Windows 8 install instructions.

### Startup

The base artifact provides a `start.sh` script to start the VM on unix-like
systems and `start.bat` for Windows. Running this script will open a graphical
console on the host machine, and create a virtualized network interface.
On Linux you may need to run with `sudo` to start the VM. If the VM does not
start then check `Debugging.md`

Once the VM has started you can login to the guest system from the host.
Whenever you are asked for a password, the answer is `password`. The default
username is `artifact`.

```
$ ssh -p 5555 artifact@localhost
```

You can also copy files to and from the host using scp.

```
$ scp -P 5555 artifact@localhost:somefile .
```

### Shutdown

To shutdown the guest system cleanly, login to it via ssh and use

```
$ sudo shutdown now
```

### Artifact Preparation

Authors should install software dependencies into the VM image as needed,
preferably via the standard Debian package manager. For example, to install
GHC and cabal-install, login to the host and type:

```
$ sudo apt update
$ sudo apt install ghc
$ sudo apt install cabal-install
```

If you really need a GUI then you can install X as follows, but we prefer
console-only artifacts whenever possible.

```
$ sudo apt install xorg
$ sudo apt install xfce4   # or some other window manager
$ startx
```

See Debugging.md for advice on resolving other potential problems.

If your artifact needs lots of memory you may need to increase the value
of the `QEMU_MEM_MB` variable in the `start.sh` script.

When preparing your artifact, please also follow the [Submission
Guidelines](https://icfp23.sigplan.org/track/icfp-2023-artifact-evaluation#Submission-Guidelines).
